function M = sai(A, P, E, print)
%SAI SAI-Preconditioner for linear system Ax=b
% P pattern of preconditioner M
% E goalmatrix   min ||AM-E||, default: identity matrix

if nargin < 4
    print = false;
end
if nargin < 3
   E = speye(size(A)); 
end

n = size(A,1);
M = sparse(P);


% for each column
for k=1:n
    
    if print && mod((k-1), floor(sqrt(n))) == 0
       fprintf('SAI Iteration %i/%i\n', k, n); 
    end
        
    % compute optimal column M_k for starting pattern 
    M_k = zeros(n,1);
    J_k = find(P(:,k));
    e_k = E(:,k);
    M_k(J_k) = solveLSwithPattern(A, e_k, J_k);
    %r_k = A*M_k-e_k;
    
    M(:,k) = M_k;
end


end



function xJ = solveLSwithPattern(A,b,J)

%%% solve system ||Ax-b||_2 with given pattern J
%%% return x(J)

% B = A(:,J);
% I = find(any(B,2));
% Ahat = B(I,:);
% bhat = b(I);
% 
% xJ = Ahat\bhat; 
xJ = A(:,J)\b;
end