function M = sai_tril(L, P, E, eps)
%SAI_TRIL SAI-Preconditioner for lower triang matrix with I=J
% L matrix of system Lx=b, to be preconditioned
% P pattern of preconditioner M
% E goalmatrix   min ||LM-E||
% eps = lower limit for diag elements

if nargin < 3
   E = speye(size(L)); 
end
if nargin < 4
    eps = 0.0001;
end

n = size(L,1);
M = sparse(n,n);
opts.LT = true; % option for solving linear system

% for each column
for k=1:n
    % compute column M_k for given pattern
    J_k = find(P(:,k));
    e_k = full(E(J_k, k));
    
    M(J_k,k) = linsolve(full(L(J_k, J_k)), e_k, opts);
    if(abs(M(k,k)) < eps)
        M(k,k) = eps;
    end
end
end
