/*
    ======================================================================
    ======================================================================
    ==                                                                  ==
    ==  MSPAI:  Modified SPAI algorithm to comupte SParse Approximate   ==
    ==          Invers matrices.                                        ==
    ==                                                                  ==
    ==  Copyright (C)  2007, 2008, 2009 by                              ==
    ==                 Matous Sedlacek <sedlacek@in.tum.de>             ==
    ==                 Chair of Scientific Computing -- Informatics V   ==
    ==                 Technische Universität München                   ==
    ==                                                                  ==
    ==  This file is part of MSPAI.                                     ==
    ==                                                                  ==
    ==  MSPAI is free software: you can redistribute it and/or          ==
    ==  modify it under the terms of the GNU Lesser General Public      ==
    ==  License as published by the Free Software Foundation, either    ==
    ==  version 3 of the License, or (at your option) any later version.==
    ==                                                                  ==
    ==  MSPAI is distributed in the hope that it will be useful,        ==
    ==  but WITHOUT ANY WARRANTY; without even the implied warranty of  ==
    ==  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the   ==
    ==  GNU Lesser General Public License for more details.             ==
    ==                                                                  ==
    ==  You should have received a copy of the GNU Lesser General       ==
    ==  Public License along with MSPAI.                                ==
    ==  If not, see <http://www.gnu.org/licenses/>.                     ==
    ==                                                                  ==
    ======================================================================
    ======================================================================
*/


//file includings
#include "Command_Line.h"


//C++ includings
#include <iostream>
#include <stdexcept>
#include <string>
#include <sstream>
#include <string.h>
#include <algorithm>



Command_Line::Command_Line()
{
    options =
            "\n"
            "\t  -Ce:  file in matrix market format containing TRANSPOSED\n"
            "\t\tprobing vectors for system matrix C.\n"
            "\t\tNotice that probing is supported only in static MSPAI without any\n"
            "\t\timprovement steps.\n"
            "\t\tdefault is no file\n"
            "\n"
            "\t  -Be:  file in matrix market format containing TRANSPOSED\n"
            "\t\tprobing vectors for target matrix B.\n"
            "\t\tNotice that probing is supported only in static MSPAI without any\n"
            "\t\timprovement steps.\n"
            "\t\tdefault is no file\n"
            "\n"
            "\t  -ep:  epsilon parameter for SPAI\n"
            "\t\tdefault is ep = 0.4\n"
            "\n"
            "\t  -mf:  message file for warning messages\n"
            "\t\tdefault is no file\n"
            "\n"
            "\t  -mn:  maximum number of new nonzero candidate per step\n"
            "\t\tdefault is mn = 5\n"
            "\n"
            "\t  -ns:  maximum number of improvement steps per row in SPAI\n"
            "\t\tdefault is ns = 5\n"
            "\n"
            "\t  -wp:  write preconditioner in matrix market format to file precond.mm \n"
            "\t\tdefault is vb = 1\n"
            "\n"
            "\t  -up:  pattern file in MatrixMarket pattern format or \n"
            "\t\t1: for diagonal pattern without file or\n"
            "\t\t2: for pattern filled where A has nnz without file\n"
            "\t\tdefault is no upper pattern\n"
            "\n"
            "\t  -cs:  cache size for runtime-optimization within SPAI. Allowed are:\n"
            "\t\t0: not using cache (default)\n"
            "\t\t0 < cs < 1e+7 for using cache with cs-size.\n"
            "\t\tdefault is cs = 0.\n"
            "\n"
            "\t  -qr:  Use QR-Updates for runtime-optimization within SPAI. Allowed are:\n"
            "\t\t0: Don't use QR-Updates within SPAI\n"
            "\t\t1: Automatic switch between dense and sparse QR-Decompositions\n"
            "\t\t   by means of the fill grade of the submatrices.\n"
            "\t\t   Fillgrade parameter will be used.\n"
            "\t\t2: dense QR-Update optimization (default).\n"
            "\t\t3: sparse QR-Update optimization.\n"
            "\t\t4: hybrid QR-Update optimization.\n"
            "\t\t5: Run QR-Code without updates and only with sparse decompositions.\n"
            "\t\t   This option is usefull for comparison with other QR modes.\n"
	        "\t\tdefault is qr = 0\n"
            "\n"
            "\t  -fg:  Use fillgrade for option \'-qr 1\'. Allowed are:\n"
            "\t\t0.0 <= fg <= 1.0\n"
            "\t\tdefault is fg = 0.3\n"
            "\n"
            "\t  -um: Use mean value as bound for augmenting indices.\n"
            "\t\tdefault is um = 1\n"
            "\n"
            "\t  -schur: Use schur probing with probing files. Allowed are:\n"
            "\t\t0: Don't use schur probing (default)\n"
            "\t\t1: Use schur probing\n"
            "\n"
            "\t  -rho: Weight factor rho for probing conditions. Allowed are:\n"
            "\t\t0.0 <= rho\n"
            "\t\tdefault is rho = 1.0\n"
            "\n"
            "\t  -ch: Using hashtable for runtime optimization in MSPAI. \n"
            "\t\tThis hashtable is used for comparison with the caching algorithm.\n"
            "\t\tIts the caching algorithm without fixed cachesize. Allowed are:\n"
            "\t\t0: not using hashtable (default)\n"
            "\t\t1: using hashtable\n"
            "\n"
            "\t  -out: Output file name.\n"
            "\t\tStore preconditioner here.\n"
            "\t\tDefault is out = ./precond.mtx\n"
            ;
}



void
Command_Line::Print_Short_License()
{        std::cout << "\n\t==========================================="
                      "\n\n\tMSPAI Copyright (C) 2007, 2008, 2009 by \n"
                      "\tMatous Sedlacek. This program comes with\n"
                      "\tABSOLUTELY NO WARRANTY. This is free soft-\n"
                      "\tware, and you are welcome to redistribute\n"
                      "\tit under certain conditions; See the\n"
                      "\tCOPYING and COPYING.LESSER file for details."
                   << std::endl;
}



void
Command_Line::Parameters(int    argc,
                         char   *argv[],
                         char   *&matrix_file,
                         char   *&pattern_file,
                         int    &pp,
                         char   *&u_pattern_file,
                         int    &up,
                         double &ep,
                         char   *&mf,
                         int    &mn,
                         int    &ns,
                         int    &wp,
                         int    &opt,
                         int    &cs,
                         int    &qr,
                         char   *&tr,
                         double &fg,
                         bool   &um,
                         char   *&probing_Ce_file,
                         char   *&probing_Be_file,
                         bool   &pr,
                         char   *&target_file,
                         int    &tp,
                         double &rho,
                         int    &us,
                         char   *&output_file)
{

    int                 k = 0,
                        ch;

    static  char        m_file[128],
                        p_file[128],
                        u_p_file[128],
                        prob_Ce_file[128],
                        prob_Be_file[128],
                        targ_file[128],
                        o_file[128],
                        tr_mode[128];

    std::stringstream   out_str;


    //set default values
    ep = 0.4;
    mf = (char*)"";
    mn = 5;
    ns = 5;
    wp = 1;
    cs = 0;
    qr = 0;
    tr = (char*)"";
    fg = 0.3;
    rho = 1.0;
    um = true;
    pr = false;
    up = 3;
    us = 0;
    ch = 0;
    output_file = (char*)"precond.mtx";

    // User needs help?
    if (( argc > 1 ) && ( strncmp(argv[1], "-h", 2) == 0 ))
    {
        throw std::runtime_error(
            "\n   ================================= HELP ===================================\n"
            "\n\tinput parameters (first, second and third are mandatory):\n\n"
            "\tsystem matrix:\n"
            "\t\tmatrix file in MatrixMarket format\n\n"
            "\tstart pattern:\n"
            "\t\tpattern file in MatrixMarket pattern format or \n"
            "\t\t-1 for diagonal pattern without file or\n"
            "\t\t-2 for pattern filled where A has nnz without file\n\n"
            "\ttarget matrix:\n"
            "\t\tmatrix file in MatrixMarket format or \n"
            "\t\t-1 for identity matrix I \n\n"
            "\t\tNotice that target matrices different to an identity matrix \n"
            "\t\tare only supported without any improvement steps in static MSPAI!\n\n\n"
            "\toptions (any order, always -code number/path, e.g. -ep 0.7):\n\n"
            "\t" + std::string(options) + "\n"
            "\n   ==========================================================================\n"
                                );
    }

    //User inputed less than 3 Parameter -> help required
    if (argc < 4)
    {
        throw std::runtime_error(
            "\n\tusage:\t./mspai-1.2-serial matrix.mm [pattern.mm |-1| -2]"
                        "[targetmatrix.mm | -1] options\n\n"
            "\tor:\t./mspai-1.2-serial -h(elp) for help on input parameters\n\n");
    }


    strcpy(m_file,argv[++k]);
    matrix_file = m_file;
    argc--;

    //Testing for start pattern
    if (argv[2][0] != '-')
    {
        pp = 0;
        strcpy(p_file,argv[++k]);
        pattern_file = p_file;
    }
    else
    {
        k++;
        if (argv[2][1] == '1')
            pp = 1;
        else if (argv[2][1] == '2')
        {
            pp = 2;
            pattern_file = matrix_file;
        }
        else
            throw std::runtime_error(
                "\n\tIllegal start pattern parameter: " + std::string(argv[k]) + "\n"
                "\tAllowed are: \n"
                "\t\tarbitrary pattern file (no number required)\n"
                "\t\t-1 for diagonal pattern (no file required)\n"
                "\t\t-2 for pattern filled where A has nnz (no file required)\n"
                "\n\tUse -h(elp) for details.\n");
    }


    //Testing for target matrix
    if (argv[3][0] != '-')
    {
        tp = 0;
        strcpy(targ_file,argv[++k]);
        target_file = targ_file;
    }
    else
    {
        k++;
        if (argv[3][1] == '1')
            tp = 1;
        else
            throw std::runtime_error(
                "\n\tIllegal target matrix parameter: " + std::string(argv[k]) + "\n"
                "\tAllowed are: \n"
                "\t\tarbitrary target matrix file (no number required)\n"
                "\t\t-1 for identity matrix (no file required)\n"
                "\n\tUse -h(elp) for details.\n");
    }


    argc--;

    while (argc-- > 3)
    {
        switch (Match_Arg(argv[++k]))
        {
            case 0:     // -ep
                ep = std::atof(argv[++k]);
                argc--;
                break;

            case 1:     //-mf
                strcpy(mf,argv[++k]);
                argc--;
                break;

            case 2:     // -mn
                mn = std::atoi(argv[++k]);
                argc--;
                break;

            case 3:     // -ns
                ns = std::atoi(argv[++k]);
                argc--;
                break;

            case 4:     // -wp
                wp = std::atoi(argv[++k]);
                argc--;
                break;

            case 5:     // -cs
                cs = std::atoi(argv[++k]);
                argc--;
                break;

            case 6:     // -qr
                qr = std::atoi(argv[++k]);
                argc--;
                break;

            case 7:     // -fg
                fg = std::atof(argv[++k]);
                argc--;
                break;

            case 8:     // -um
                um = std::atoi(argv[++k]);
                argc--;
                break;

            case 9:     // -schur
                us = std::atoi(argv[++k]);
                argc--;
                break;

            case 10:     // -Ce
                strcpy(prob_Ce_file,argv[++k]);
                probing_Ce_file = prob_Ce_file;
                pr = true;
                argc--;
                break;

            case 11:     // -Be
                strcpy(prob_Be_file,argv[++k]);
                probing_Be_file = prob_Be_file;
                pr = true;
                argc--;
                break;

            case 12:     // -up
                strcpy(u_p_file,argv[++k]);
                u_pattern_file = u_p_file;
                up = 0;
                argc--;
                break;

            case 13:     // -rho
                rho = std::atof(argv[++k]);
                argc--;
                break;

            case 14:     // -ch
                ch = std::atoi(argv[++k]);;
                argc--;
                break;

            case 15:     // -out (output file)
                strcpy(o_file, argv[++k]);
                output_file = o_file;
                argc--;
                break;
            case 16:     // -tr
                strcpy(tr_mode, argv[++k]);
                tr = tr_mode;
                argc--;
                break;

            default:    //Illegal parameter
                throw std::runtime_error(
                    "\n\tIllegal parameter: " + std::string(argv[k]) + "\n"
                    "\tCheck your input please.\n");
                k++;
                argc--;
        }
    }

    //check for correct number of steps
    if (ns < 0)
    {
        out_str << ns;
        throw std::runtime_error(
            "\n\tIllegal number of improvement steps: "
            + out_str.str() +  "\n"
            "\tAllowed are positive values.\n");
    }

    //check for correct cache size
    if (cs < 0 || cs > 1e7)
    {
        out_str << cs;
        throw std::runtime_error(
            "\n\tIllegal cache size parameter: "
                + out_str.str() + "\n"
            "\tAllowed are positiv values smaller than 1e+7.\n"
            "\tdefault is cs = 0\n");
    }

    //check for correct cache size
    if (ch < 0 || ch > 1)
    {
        out_str << ch;
        throw std::runtime_error(
            "\n\tIllegal hash size parameter: "
                + out_str.str() + "\n"
            "\tPlease use 1 for using a hashtable and 0 for not using a hashtable.\n"
            "\tdefault is ch = 0\n");
    }

    if (cs > 0 && ch > 0)
    {
        throw std::runtime_error(
            "\n\tCaching and Hashing cannot be used simultaneously\n"
            "\tPlease use either caching OR hashing algorithm.\n");
    }

    // QR updates and caching
    // cannot be used at once
    // at the moment
    if (cs > 0 && qr > 0)
    {
        throw std::runtime_error(
            "\n\tQR-Updates and Caching optimization cannot\n"
            "\tbe used simultaneously!\n"
            "\tPlease decide whether you want to use\n"
            "\tQR-Updates OR Caching.\n");
    }

    // QR updates and hashing
    // cannot be used at once
    // at the moment
    if (ch > 0 && qr > 0)
    {
        throw std::runtime_error(
            "\n\tQR-Updates and Hashing optimization cannot\n"
            "\tbe used simultaneously!\n"
            "\tPlease decide whether you want to use\n"
            "\tQR-Updates OR Hashing.\n");
    }

    if (qr < 0 || qr > 5)
    {
        out_str << qr;
        throw std::runtime_error(
            "\n\tIllegal parameter for qr: "
            + out_str.str() +  "\n"
            "\tAllowed are:\n"
            "\t\t0: Don't use QR-Updates within SPAI (default)\n"
            "\t\t1: Automatic switch between dense and hybrid QR-Updates\n"
            "\t\t   by means of the fill grade of the submatrices.\n"
            "\t\t   Fillgrade parameter will be used.\n"
            "\t\t2: dense QR-Update optimization within SPAI.\n"
            "\t\t3: sparse QR-Update optimization within SPAI.\n"
            "\t\t4: hybrid QR-Update optimization within SPAI.\n"
            "\t\t5: no updates, but sparse decompositions.\n"
            "\t\t   This option is useful for comparison with other QR modes.\n");
    }

    if (fg < 0.0 || fg > 1.0)
    {
        out_str << fg;
        throw std::runtime_error(
            "\n\tIllegal parameter for fg: "
            + out_str.str() +  "\n"
            "\tAllowed are:\n"
            "\t\t0.0 <= fg <= 1.0\n"
            "\t\tdefault is fg = 0.3\n");
    }

    if (rho < 0.0)
    {
        out_str << rho;
        throw std::runtime_error(
            "\n\tIllegal parameter for rho: "
            + out_str.str() +  "\n"
            "\tAllowed are:\n"
            "\t\t0.0 <= rho\n"
            "\t\tdefault is rho = 1.0\n");
    }

    if (um < 0 || um > 1)
    {
        out_str << um;
        throw std::runtime_error(
            "\n\tIllegal value for using mean value: "
                + out_str.str() + "\n"
            "\tAllowed are:\n"
            "\t\t0: don't use mean value as bound for augmenting indices\n"
            "\t\t1: use mean value as bound for augmenting indices (default)\n");
    }

    if (us < 0 || us > 1)
    {
        out_str << us;
        throw std::runtime_error(
            "\n\tIllegal value for using schur value: "
                + out_str.str() + "\n"
            "\tAllowed are:\n"
            "\t\t0: don't use schur probing (default) \n"
            "\t\t1: use schur probing \n");
    }

    if (us && !tp)
    {
        out_str << us;
        throw std::runtime_error(
            "\n\tArbitrary target matrices are not\n"
            "\tallowed when using schur probing!\n"
            "\tPlease use the target matrix option \"-1\"\n"
            "\tso the necessary schur probing matrix B of \n"
            "\tthe form (0 I)^T can be created internally.\n");
    }

    if (us && pp > 0)
        throw std::runtime_error(
            "\n\tNo identity pattern or pattern filled where A has\n"
            "\tnnz is allowed when using schur probing!\n"
            "\tPlease verify not to use the start pattern\n"
            "\toptions \"-1\" or \"-2\"!\n"
            "\tUse a valid start pattern file instead.\n");


    if (up < 3)
    {
        if (u_pattern_file[0] == '1') up = 1;
        else if (u_pattern_file[0] == '2')
        {
            up = 2;
            u_pattern_file = matrix_file;
        }
        if (up < 3 && us)
            throw std::runtime_error(
                "\n\tUsing an upper pattern is not possible when\n"
                "\tusing schur probing!\n");
    }


    // Check if user wants to make probing/targeting
    // with dynamic pattern updates - this is not supported at the
    // MSPAI 1.0 version so beat it!
    if ((pr || !tp) && (ns > 0 || qr > 0))
        throw std::runtime_error(
            "\n\tProbing constraints and/or target matrix approximations\n"
            "\tare only supported in static MSPAI yet!\n"
            "\tPlease set the number of improvement steps to 0.\n"
            "\tQR-Updates are only possible for a dynamic SPAI.\n");


    if ((probing_Ce_file && !probing_Be_file) ||
         (!probing_Ce_file && probing_Be_file))
        throw std::runtime_error(
            "\n\tOnly one probing condition defined!\n"
            "\tPlease set a probing condition for C (-Ce)\n"
            "\tand a probing condition for B (-Be)!\n");

    // Check if user does not use any probing files and wants schur probing
    if (!probing_Ce_file && !probing_Be_file && us)
        throw std::runtime_error(
            "\n\tNo probing condition defined but schur \n"
            "\tprobing requested!\n"
            "\tPlease set a probing condition for C (-Ce)\n"
            "\tand a probing condition for B (-Be)!\n");

    // Check if user uses correct parameter "U" or "L" if solving a triangular system
    if (strcmp(tr,"") != 0 && strcmp(tr,"L") != 0 && strcmp(tr,"U") != 0)
    {
        out_str << tr;
        throw std::runtime_error(
            "\n\tIllegal parameter for tr: "
            + out_str.str() +  "\n"
            "\tAllowed are:\n"
            "\t\t0: Don't use the triangular algorithm (default)\n"
            "\t\t1: -tr \"L\": Use the solver for lower triangular matrices\n"
            "\t\t2: -tr \"U\": Use the solver for lower triangular matrices\n");
    }

    //Get optimization mode
    opt = 0;
    if (cs > 0)
        opt = 1;
    if (ch > 0)
        opt = 2;
    if (qr > 0)
        if(ns > 0 && mn > 0) // no update-steps -> no qr udpates
            opt = 3;
    if (strcmp(tr,"") != 0)
    	opt = 4;
}



int
Command_Line::Match_Arg(char *string)
{
    int     k = 0;

    static const char*
    param_table[] = {
                    "-ep",
                    "-mf",
                    "-mn",
                    "-ns",
                    "-wp",
                    "-cs",
                    "-qr",
                    "-fg",
                    "-um",
                    "-schur",
                    "-Ce",
                    "-Be",
                    "-up",
                    "-rho",
                    "-ch",
                    "-out",
                    "-tr",
                    " "     // must remain last - ends list
                    };

    do
        if (!strcmp(string, param_table[k]))
            return k;
    while (*param_table[++k] != ' ');

    return -1;
}
