/*
    ======================================================================
    ======================================================================
    ==                                                                  ==
    ==  MSPAI:  Modified SPAI algorithm to comupte SParse Approximate   ==
    ==          Invers matrices.                                        ==
    ==                                                                  ==
    ==  Copyright (C)  2007, 2008, 2009 by                              ==
    ==                 Matous Sedlacek <sedlacek@in.tum.de>             ==
    ==                 Chair of Scientific Computing -- Informatics V   ==
    ==                 Technische Universität München                   ==
    ==                                                                  ==
    ==  This file is part of MSPAI.                                     ==
    ==                                                                  ==
    ==  MSPAI is free software: you can redistribute it and/or          ==
    ==  modify it under the terms of the GNU Lesser General Public      ==
    ==  License as published by the Free Software Foundation, either    ==
    ==  version 3 of the License, or (at your option) any later version.==
    ==                                                                  ==
    ==  MSPAI is distributed in the hope that it will be useful,        ==
    ==  but WITHOUT ANY WARRANTY; without even the implied warranty of  ==
    ==  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the   ==
    ==  GNU Lesser General Public License for more details.             ==
    ==                                                                  ==
    ==  You should have received a copy of the GNU Lesser General       ==
    ==  Public License along with MSPAI.                                ==
    ==  If not, see <http://www.gnu.org/licenses/>.                     ==
    ==                                                                  ==
    ======================================================================
    ======================================================================
*/


#ifndef GUARD_JACOBI_H
#define GUARD_JACOBI_H

//A++ includings
#include "math.h"
#include "Cs.h"


///////////////////////////////////////////////////
///     \class Jacobi
///     \brief  This class holds the Jacobi implementation
///////////////////////////////////////////////////
template <class T>
class Jacobi
{
    public:

        /// Destructor
        virtual     ~Jacobi(){ }

        ///////////////////////////////////////////////////////////
		///     \brief  This function computes ||A_REAL *  M_REAL - B_REAL||
		///////////////////////////////////////////////////////////
        double 		calculate_error (  Matrix<double>*        A_REAL,
									   Matrix<double>*        M_REAL,
									   Matrix<double>*        B_REAL);

        ///////////////////////////////////////////////////////////
		///     \brief  This function does Jacobi
		///////////////////////////////////////////////////////////
        cs*          jacobi         (  Matrix<double>*        A_REAL,
									   Matrix<double>*        M_REAL,
									   double                 eps,
									   int					  max_iter);

};

#include "Jacobi.imp"

#endif
