/*
    ======================================================================
    ======================================================================
    ==                                                                  ==
    ==  MSPAI:  Modified SPAI algorithm to comupte SParse Approximate   ==
    ==          Invers matrices.                                        ==
    ==                                                                  ==
    ==  Copyright (C)  2007, 2008, 2009 by                              ==
    ==                 Matous Sedlacek <sedlacek@in.tum.de>             ==
    ==                 Chair of Scientific Computing -- Informatics V   ==
    ==                 Technische Universität München                   ==
    ==                                                                  ==
    ==  This file is part of MSPAI.                                     ==
    ==                                                                  ==
    ==  MSPAI is free software: you can redistribute it and/or          ==
    ==  modify it under the terms of the GNU Lesser General Public      ==
    ==  License as published by the Free Software Foundation, either    ==
    ==  version 3 of the License, or (at your option) any later version.==
    ==                                                                  ==
    ==  MSPAI is distributed in the hope that it will be useful,        ==
    ==  but WITHOUT ANY WARRANTY; without even the implied warranty of  ==
    ==  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the   ==
    ==  GNU Lesser General Public License for more details.             ==
    ==                                                                  ==
    ==  You should have received a copy of the GNU Lesser General       ==
    ==  Public License along with MSPAI.                                ==
    ==  If not, see <http://www.gnu.org/licenses/>.                     ==
    ==                                                                  ==
    ======================================================================
    ======================================================================
*/


//file includings
#include "Read_mm_Matrix.h"
#include "Command_Line.h"
#include "Matrix.h"
#include "Spai.h"
#include "Pattern.h"
#include "Switch_Algorithm.h"
#include "Pattern_Switch.h"
#include "Macros.h"
#include "Jacobi.h"

//C++ includings
#include <iostream>
#include <stdexcept>


int main(int argc, char *argv[])
{
  
    // SPAI parameters 
    double          epsilon_param,
                    fillgrade_param,
                    rho_param;
    
    char            *message_file       = NULL,
                    *matrix_file        = NULL,
                    *pattern_file       = NULL,
                    *u_pattern_file     = NULL,
                    *probing_Ce_file    = NULL,
                    *probing_Be_file    = NULL,
                    *target_file        = NULL,
                    *output_file        = NULL,
    				*tr                 = NULL;
    
    int             maxnew_param,
                    max_impr_steps,
                    write_param,
                    pattern_param,
                    u_pattern_param,
                    cache_param,
                    opt_level,
                    qr,
                    prob_Ce_N = -1,
                    target_param,
                    use_schur;
    
    bool            symmetric_system = false,
                    hermitian_system = false,
                    use_mean,
                    use_prob;

            
    Matrix<double>  *A_REAL    = NULL;
    Matrix<double>  *B_REAL    = NULL;
    Matrix<double>  *M_REAL    = NULL;
            
    Matrix<COMPLEX> *A_COMPLEX = NULL;
    Matrix<COMPLEX> *B_COMPLEX = NULL;
    Matrix<COMPLEX> *M_COMPLEX = NULL;
            
    Pattern         *P         = NULL,
                    *UP        = NULL;
    
            
    try //new will throw its own exception - just make one catch at all
    {
        Command_Line o_cm;
        o_cm.Print_Short_License();
        std::cout << "\n\t===========================================" << std::endl;
        std::cout << "\t===========   STARTING MSPAI   ============\n" << std::endl;
        
        try 
        { 
            // Getting the user input parameters from shell            
            std::cout << "\t* Reading input parameters..." << std::endl;
            o_cm.Parameters(argc, 
                            argv, 
                            matrix_file, 
                            pattern_file,
                            pattern_param,
                            u_pattern_file,
                            u_pattern_param,
                            epsilon_param, 
                            message_file, 
                            maxnew_param, 
                            max_impr_steps, 
                            write_param,
                            opt_level,
                            cache_param,
                            qr,
                            tr,
                            fillgrade_param,
                            use_mean,
                            probing_Ce_file,
                            probing_Be_file,
                            use_prob,
                            target_file,
                            target_param,
                            rho_param,
                            use_schur,
                            output_file); 
            
            Read_mm_Matrix o_rm;
            
            //Using switch-statement for possible enhancements
            switch(o_rm.Get_Matrix_Type(matrix_file, 
                   symmetric_system, hermitian_system))
            {   
                case real:  
                {
                    std::cout << "\t    Input is REAL!\n" << std::endl;
                    if (symmetric_system)
                    {
                        std::cout << "\t    WARNING: Matrix is symmetric but\n";
                        std::cout << "\t\t     in general MSPAI will not\n";
                        std::cout << "\t\t     preserve symmetry.\n";
                        std::cout << std::endl;
                    }

                    // Check for optimization level and get the requested algorithm
                    // Invoking Template Method Pattern
                    std::cout << "\t* Checking opimization level..." << std::endl;
                    Switch_Algorithm<double> o_alg;
                    Spai<double>* alg_ptr  = o_alg.Get_Algorithm(opt_level,
                                                                 cache_param,
                                                                 qr,
                                                                 tr,
                                                                 fillgrade_param);

                    if(false) {
						// Reading data input and generating the matrix C
						std::cout << "\t* Reading system matrix data..." << std::endl;
						o_rm.Read_Matrix_File(matrix_file,
											  probing_Ce_file,
											  use_prob,
											  use_schur,
											  symmetric_system,
											  hermitian_system,
											  A_REAL,
											  prob_Ce_N,
											  rho_param);



						if (!target_param || use_prob)
						{
							// Reading data input and generating the
							// target matrix B
							std::cout << "\t* Reading target matrix data..."
									  << std::endl;
							o_rm.Read_Target_File(A_REAL,
												  target_file,
												  probing_Be_file,
												  use_prob,
												  use_schur,
												  B_REAL,
												  prob_Ce_N,
												  target_param,
												  rho_param);
						}

						// Reading pattern file and generating the Pattern
						std::cout << "\t* Generating pattern data..." << std::endl;
						Pattern_Switch<double> o_ps;
						P = o_ps.Generate_Pattern(A_REAL,
												  pattern_file,
												  pattern_param,
												  use_schur,
												  prob_Ce_N,
												  use_prob);


						// If user wants upper pattern then make it
						if (u_pattern_param < 3)
						{
							// Reading upper pattern file and generating
							// the Upper Pattern
							std::cout << "\t* Generating upper pattern data..."
									  << std::endl;
							UP = o_ps.Generate_Pattern(A_REAL,
													   u_pattern_file,
													   u_pattern_param,
													   use_schur,
													   prob_Ce_N,
													   use_prob);
						}

	                    // Compute the preconditioner with SPAI algorithm
	                    // for real matrices
	                    std::cout << "\t* Computing SPAI..." << std::endl;
	                    alg_ptr->SPAI_Algorithm(A_REAL,
	                                            M_REAL,
	                                            B_REAL,
	                                            P,
	                                            UP,
	                                            epsilon_param,
	                                            maxnew_param,
	                                            max_impr_steps,
	                                            use_mean);
                    }
                    else {
                    	//TODO: Important: -ns must be set to 0 if we use a B! Check if this condition is true.
						// Reading data input and generating the matrix B
						std::cout << "\t* Reading system matrix data..." << std::endl;
						o_rm.Read_Matrix_File(matrix_file,
											  probing_Ce_file,
											  use_prob,
											  use_schur,
											  symmetric_system,
											  hermitian_system,
											  B_REAL,
											  prob_Ce_N,
											  rho_param);

						// compute B^T
						Matrix<double> *B_REAL_TRANSPOSED = NULL;
						B_REAL->transpose(B_REAL, B_REAL_TRANSPOSED);

						//B_REAL_TRANSPOSED->Print_Matrix_Human_Readable(B_REAL_TRANSPOSED,B_REAL_TRANSPOSED->m,B_REAL_TRANSPOSED->n);
						//B_REAL_TRANSPOSED->Print_Matrix_For_Matlab(B_REAL_TRANSPOSED,B_REAL_TRANSPOSED->m,B_REAL_TRANSPOSED->n);


						// compute L=tril(B_REAL) and U=triu(B_REAL) and L^T
						Matrix<double> *L = NULL;
						Matrix<double> *U = NULL;
						Matrix<double> *L_transposed = NULL;
						Matrix<double> *U_transposed = NULL;
						B_REAL->trilu(B_REAL,L,U);
						L->transpose(L, L_transposed);

						// get patterns for tril and triu
						Pattern* P_tril_transposed = L->To_Pattern(L,false);
						Pattern* P_triu = U->To_Pattern(U,false);

						for (int i=0; i<1; i++) {
							if(i % 2 == 0) {
			                    alg_ptr->SPAI_Algorithm(L,
			                                            U,
			                                            B_REAL,
			                                            P_triu,
			                                            UP,
			                                            epsilon_param,
			                                            maxnew_param,
			                                            max_impr_steps,
			                                            use_mean);
							}
							else {
			                    alg_ptr->SPAI_Algorithm(B_tril,
			                                            B_triu,
			                                            B_REAL,
			                                            P_triu,
			                                            UP,
			                                            epsilon_param,
			                                            maxnew_param,
			                                            max_impr_steps,
			                                            use_mean);
							}
		                    // Compute the preconditioner with SPAI algorithm
		                    // for real matrices
		                    std::cout << "\t* Computing SPAI..." << std::endl;

						}

						// print output just for testing
						B_REAL->Print_Matrix_Human_Readable(B_REAL,B_REAL->m,B_REAL->n);
						B_REAL->Print_Matrix_For_Matlab(B_REAL,B_REAL->m,B_REAL->n);

						B_tril->Print_Matrix_Human_Readable(B_tril,B_tril->m,B_tril->n);
						B_tril->Print_Matrix_For_Matlab(B_tril,B_tril->m,B_tril->n);
						B_triu->Print_Matrix_Human_Readable(B_triu,B_triu->m,B_triu->n);
						B_triu->Print_Matrix_For_Matlab(B_triu,B_triu->m,B_triu->n);
						//P->Print_Pattern_Data();
						//TODO: tril and triu must be deleted (freed) somewhen in the future
					}
                    


                    M_REAL->Print_Matrix_Human_Readable(M_REAL,M_REAL->m,M_REAL->n);
                    M_REAL->Print_Matrix_For_Matlab(M_REAL,M_REAL->m,M_REAL->n);


                    // Do Jacobi
                    Jacobi<double>* jacobi  = new Jacobi<double>();
                    if (true) {
                    	std::cout << "\t* Doing Jacobi..." << std::endl;
                    	double epsilon = 10e-4;
                    	int max_iter = 100;
                    	jacobi->jacobi(A_REAL, M_REAL, epsilon, max_iter);
                    }

                    // output the absolute error: ||A_REAL *  M_REAL - B_REAL||
                    if (true) {
                    	std::cout << "\t* Computing Error..." << std::endl;
                    	double error = jacobi->calculate_error(A_REAL,
																M_REAL,
																B_REAL);
                        std::cout << "\t    The error ||A *  M - B|| in the 1-norm is: " <<  error << "\n" << std::endl;
                    }
                    
                    if (write_param)
                    {
                        // Write preconditioner to file
                        std::cout << "\t* Writing solution to file " +
                                     std::string(output_file)
                                  << std::endl;
                        M_REAL->Write_Matrix_To_File(M_REAL, output_file);
                    }

                    delete alg_ptr;
                    delete A_REAL;  
                    delete B_REAL;
                    delete M_REAL;
                    
                    break;
                }                   
                case complex:
                {               
                    // Check if user want to run SPAI with 
                    // QR-Updates with COMPLEX input
                    // This is not supported yet! Abort Program.
                    if (qr) 
                        throw std::runtime_error(
                            "\n\t\tQR-Update optimization is supported only for \n"
                            "\t\treal and not for COMPLEX matrices!\n"
                            "\t\tPlease change your input.\n");                     
                    
                    std::cout << "\t    Input is COMPLEX!\n" << std::endl;
                    
                    if (symmetric_system)
                    {
                            std::cout << "\t    WARNING: Matrix is symmetric but\n";
                            std::cout << "\t\t     in general MSPAI will not\n";
                            std::cout << "\t\t     preserve symmetry.\n";
                            std::cout << std::endl;
                    }

                    // Reading data input and generating the matrix A
                    std::cout << "\t* Reading matrix data..." << std::endl;
                    o_rm.Read_Matrix_File(matrix_file, 
                                          probing_Ce_file, 
                                          use_prob, 
                                          use_schur,
                                          symmetric_system, 
                                          hermitian_system,
                                          A_COMPLEX,
                                          prob_Ce_N,
                                          rho_param);
                 
                    if (!target_param || use_prob)
                    {
                        // Reading data input and generating the target matrix B
                        std::cout << "\t* Reading target matrix data..." 
                                  << std::endl;
                        o_rm.Read_Target_File(A_COMPLEX,
                                              target_file, 
                                              probing_Be_file, 
                                              use_prob, 
                                              use_schur,
                                              B_COMPLEX,
                                              prob_Ce_N,
                                              target_param,
                                              rho_param);
                    }
                    
                    // Reading pattern file and generating the Pattern
                    std::cout << "\t* Generating pattern data..." << std::endl;
                    Pattern_Switch<COMPLEX> o_ps;
                    P = o_ps.Generate_Pattern(A_COMPLEX,
                                              pattern_file,
                                              pattern_param,
                                              use_schur,
                                              prob_Ce_N,
                                              use_prob);
                    
                    
                    // If user wants upper pattern then make it
                    if (u_pattern_param < 3)
                    {
                        // Reading upper pattern file and generating the 
                        // Upper Pattern
                        std::cout << "\t* Generating upper pattern data..." 
                                  << std::endl;
                        UP = o_ps.Generate_Pattern(A_COMPLEX,
                                                   u_pattern_file,
                                                   u_pattern_param,
                                                   use_schur,
                                                   prob_Ce_N,
                                                   use_prob);
                    }
                    
                    
                    // Check for optimization level and get the requested algorithm
                    // Invoking Template Method Pattern
                    std::cout << "\t* Checking opimization level..." << std::endl;
                    Switch_Algorithm<COMPLEX> o_alg;
                    Spai<COMPLEX>* alg_ptr  = o_alg.Get_Algorithm(opt_level, 
                                                                  cache_param, 
                                                                  qr,
                                                                  tr,
                                                                  fillgrade_param);
                    
                    
                    // Compute the preconditioner with SPAI 
                    // algorithm for complex matrices
                    std::cout << "\t* Computing SPAI..." << std::endl;
                    alg_ptr->SPAI_Algorithm(A_COMPLEX, 
                                            M_COMPLEX,
                                            B_COMPLEX,
                                            P,
                                            UP,
                                            epsilon_param,
                                            maxnew_param,
                                            max_impr_steps,
                                            use_mean);
                    
                    
                    if (write_param)
                    {
                        // Write preconditioner to file
                        std::cout << "\t* Writing solution to file " +
                                     std::string(output_file)
                                  << std::endl;
                        M_COMPLEX->Write_Matrix_To_File(M_COMPLEX, output_file);
                    }
                    
                    delete alg_ptr;
                    delete A_COMPLEX;   
                    delete B_COMPLEX;
                    delete M_COMPLEX;
                    
                    break;      
                }           
                //default case will not occur
            }   
        } 
        catch (std::runtime_error& ex) 
        {                   
            delete A_REAL;  
            delete B_REAL;
            delete M_REAL;
            delete A_COMPLEX; 
            delete B_COMPLEX;  
            delete M_COMPLEX;
            delete P;
            delete UP;
            std::cerr << COLOR_RED << ex.what() << COLOR_NORMAL << std::endl;
            std::cerr << "\t===========================================\n" 
                      << std::endl;
            return EXIT_FAILURE;
        }
    }
    catch(std::bad_alloc &e) 
    {
        delete A_REAL;
        delete B_REAL;  
        delete M_REAL;
        delete A_COMPLEX; 
        delete B_COMPLEX;  
        delete M_COMPLEX;
        delete P;
        delete UP;
        std::cerr << COLOR_RED <<"\n\tPROGRAM ABORTED: " 
                  << e.what() << "\n\t\t  ERROR: Not enough memory.\n" 
                  "\n\t\t  Try to use a diagonal start pattern"
                  "\n\t\t  and/or reduce the number of pattern"
                  "\n\t\t  updates and indices to be added per"
                  "\n\t\t  step.\n" << COLOR_NORMAL << std::endl;
        std::cerr << "\t===========================================\n" << std::endl;
        return EXIT_FAILURE;
    }

    delete P;
    delete UP;
                
    std::cout << "\n\t========   SUCCESSFULLY FINISHED   ========" << std::endl;
    std::cout << "\t===========================================\n" << std::endl;
    return EXIT_SUCCESS;
}
